/*
* ESP32 WebServer (Station Mode) Example: button and slider for LED control
* Use Board: Wrover Module
* Full Tutorial @ https://deepbluembedded.com/esp32-wifi-library-examples-tutorial-arduino/
*
* 4/17/23 dje Mods for Multizone Stereo Control. Learning some javascript and html
*         Added 8 zone and 4 source buttons
*         Highlighting selected button, torn off other buttons. Radio Buttons the hard way. 
*         Removed slider
*         Sources: CD, PHONO, TUN, TV, PC, DVD, MP3, AUX
*         Zones: LR, DEN, BR1, BR2, BASE, DECK, KIT, BATH 
*         Controls: v, V, q, Q
* 4/19/23 Send control character to multizone on Serial2. Needs TTL-RS232 driver
*
*/

#include <WiFi.h>
#include <WebServer.h>
 
// Replace with your own network credentials
const char* ssid = "davenet";
const char* password = "feedc0de69";
 
WebServer server(80);

int LED_BUILTIN = 2;

const int led1Pin = LED_BUILTIN;
const int led2Pin = 13;
int led1State = LOW;

const char html[] PROGMEM = R"rawliteral(
<!DOCTYPE html>
<html>
<head>
  <title>Multizone Web Server</title>
  <style>
    body {
      font-family: sans-serif;
    }
   .container {
   /*   display: flex; */
      flex-direction: row;
      align-items: left;
    }

    button {
      font-size: 1.5em;
      display: inline-block;
      margin: 5px;
      padding: 5px 5px;
      background-color: lightgray; /* Set the initial background color */
      border-radius: 8px;
      width: 80px;
      float: left;
      border: 2px solid black;
    }
    button.on {
      background-color: lightgreen; /* Set the background color when the LED is ON */
    }
    input[type=range] {
      width: 50%;
    }
  </style>
</head>
<body>
  <div class="container">
    <h1>Multizone Audio Control</h1>
    <h3>Zone Select</h3>
    <button id="buttonZ1" onclick="zone1()"> LR </button>
    <button id="buttonZ2" onclick="zone2()"> DEN </button>
    <button id="buttonZ3" onclick="zone3()"> BR1 </button>
    <button id="buttonZ4" onclick="zone4()"> BR2 </button>
    <br>
    <br>
    <h3> </h3>
    <button id="buttonZ5" onclick="zone5()"> BASE </button>
    <button id="buttonZ6" onclick="zone6()"> DECK </button>
    <button id="buttonZ7" onclick="zone7()"> KIT </button>
    <button id="buttonZ8" onclick="zone8()"> BATH </button>
    <br>
    <br>
    <h3>Source Select</h3>
    <button id="buttonS1" onclick="src1()"> CD </button>
    <button id="buttonS2" onclick="src2()"> PHO </button>
    <button id="buttonS3" onclick="src3()"> TUN </button>
    <button id="buttonS4" onclick="src4()"> TV </button>    
    <br>
    <br>
    <h3> </h3>
    <button id="buttonS5" onclick="src5()"> PC </button>
    <button id="buttonS6" onclick="src6()"> DVD </button>
    <button id="buttonS7" onclick="src7()"> MP3 </button>
    <button id="buttonS8" onclick="src8()"> AUX </button>
    <br>
    <br>
    <h3>Controls</h3>
    <button id="buttonV1" onclick="vol1()"> VUP </button>
    <button id="buttonV2" onclick="vol2()"> VDN </button>
    <button id="buttonV3" onclick="vol3()"> MUTE </button>
    <button id="buttonV4" onclick="vol4()"> STFU </button>
    <br>
  
  </div>
  <script>
    var zone1State = true;
    var zone2State = false;
    var zone3State = false;
    var zone4State = false;
    var zone5State = false;
    var zone6State = false;
    var zone7State = false;
    var zone8State = false;

    var src1State = false;
    var src2State = false;
    var src3State = false;
    var src4State = false;
    var src5State = false;
    var src6State = false;
    var src7State = false;
    var src8State = false;
 
    var vol1State = false;
    var vol2State = false;
    var vol3State = false;
    var vol4State = false;

    function clearZones(){
      buttonZ1.classList.toggle('on', 0);
      buttonZ2.classList.toggle('on', 0);
      buttonZ3.classList.toggle('on', 0);
      buttonZ4.classList.toggle('on', 0);
      buttonZ5.classList.toggle('on', 0);
      buttonZ6.classList.toggle('on', 0);
      buttonZ7.classList.toggle('on', 0);
      buttonZ8.classList.toggle('on', 0); 
    }
     function clearSrc(){
      buttonS1.classList.toggle('on', 0);
      buttonS2.classList.toggle('on', 0);
      buttonS3.classList.toggle('on', 0);
      buttonS4.classList.toggle('on', 0);
      buttonS5.classList.toggle('on', 0);
      buttonS6.classList.toggle('on', 0);
      buttonS7.classList.toggle('on', 0);
      buttonS8.classList.toggle('on', 0);   
    } 
    function zone1() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/zone1?state=' + (zone1State ? '1' : '0'), true);
      xhr.send();
      clearZones();
      zone1State = 1;
      buttonZ1.classList.toggle('on', zone1State);    
    }
    function zone2() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/zone2?state=' + (zone2State ? '1' : '0'), true);
      xhr.send();
      clearZones();
      zone2State = 1;
      buttonZ2.classList.toggle('on', zone2State);
    }
    function zone3() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/zone3?state=' + (zone3State ? '1' : '0'), true);
      xhr.send();
      clearZones();
      zone3State = 1;
      buttonZ3.classList.toggle('on', zone3State);
    }
    function zone4() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/zone4?state=' + (zone4State ? '1' : '0'), true);
      xhr.send();
      clearZones();
      zone4State = 1;
      buttonZ4.classList.toggle('on', zone4State);      
    }
    function zone5() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/zone5?state=' + (zone5State ? '1' : '0'), true);
      xhr.send();
      clearZones();
      zone5State = 1;
      buttonZ5.classList.toggle('on', zone5State);      
    }
    function zone6() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/zone6?state=' + (zone6State ? '1' : '0'), true);
      xhr.send();
      clearZones();
      zone6State = 1;
      buttonZ6.classList.toggle('on', zone6State);      
    }
    function zone7() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/zone7?state=' + (zone7State ? '1' : '0'), true);
      xhr.send();
      clearZones();
      zone7State = 1;
      buttonZ7.classList.toggle('on', zone7State);      
    }
    function zone8() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/zone8?state=' + (zone8State ? '1' : '0'), true);
      xhr.send();
      clearZones();
      zone8State = 1;
      buttonZ8.classList.toggle('on', zone8State);      
    }
        
    function src1() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/src1?state=' + (src1State ? '1' : '0'), true);
      xhr.send();
      clearSrc();
      src1State = 1;
      buttonS1.classList.toggle('on', src1State);  
    }
    function src2() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/src2?state=' + (src2State ? '1' : '0'), true);
      xhr.send();
      clearSrc();
      src2State = 1;
      buttonS2.classList.toggle('on', src2State);  
    }
    function src3() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/src3?state=' + (src3State ? '1' : '0'), true);
      xhr.send();
      clearSrc();
      src3State = 1;
      buttonS3.classList.toggle('on', src3State);  
    }
    function src4() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/src4?state=' + (src4State ? '1' : '0'), true);
      xhr.send();
      clearSrc();
      src4State = 1;
      buttonS4.classList.toggle('on', src4State);  
    }   
    function src5() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/src5?state=' + (src5State ? '1' : '0'), true);
      xhr.send();
      clearSrc();
      src5State = 1;
      buttonS5.classList.toggle('on', src5State);  
    }
    function src6() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/src6?state=' + (src6State ? '1' : '0'), true);
      xhr.send();
      clearSrc();
      src6State = 1;
      buttonS6.classList.toggle('on', src6State);  
    }
    function src7() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/src7?state=' + (src7State ? '1' : '0'), true);
      xhr.send();
      clearSrc();
      src7State = 1;
      buttonS7.classList.toggle('on', src7State);  
    }
    function src8() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/src8?state=' + (src8State ? '1' : '0'), true);
      xhr.send();
      clearSrc();
      src8State = 1;
      buttonS8.classList.toggle('on', src8State);  
    } 
    function vol1() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/vol1?state=' + (vol1State ? '1' : '0'), true);
      xhr.send();
    }
    function vol2() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/vol2?state=' + (vol2State ? '1' : '0'), true);
      xhr.send();
    }
    function vol3() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/vol3?state=' + (vol3State ? '1' : '0'), true);
      xhr.send();
    }
    function vol4() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/vol4?state=' + (vol4State ? '1' : '0'), true);
      xhr.send();
    } 
  </script>
</body>
</html>
)rawliteral";
 
void handleRoot() {
  server.send(200, "text/html", html);
}
 
void handleLED() {
  led1State = server.arg("state").toInt();
  digitalWrite(led1Pin, led1State);
  server.send(200, "text/plain", String(led1State));
  Serial.print("LED1: ");
  Serial.println(led1State);
}
 
// void handleBrightness() {
//   ledBrightness = server.arg("value").toInt();
//   analogWrite(led2Pin, ledBrightness);
//   server.send(200, "text/plain", String(ledBrightness));
//   Serial.print("Brightness: ");
//   Serial.println(ledBrightness);
// }
void handleZone1() {
  server.send(200, "text/plain", "zone1");
  Serial.println("Zone1");
  Serial2.print('!');
}
void handleZone2() {
  server.send(200, "text/plain", "zone2");
  Serial.println("Zone2");
  Serial2.print('@');
}
void handleZone3() {
  server.send(200, "text/plain", "zone3");
  Serial.println("Zone3");
  Serial2.print('#');
}
void handleZone4() {
  server.send(200, "text/plain", "zone4");
  Serial.println("Zone4");
  Serial2.print('$');
}
void handleZone5() {
  server.send(200, "text/plain", "zone5");
  Serial.println("Zone5");
  Serial2.print('%');
}
void handleZone6() {
  server.send(200, "text/plain", "zone6");
  Serial2.print('^');
  Serial.println("Zone6");
}
void handleZone7() {
  server.send(200, "text/plain", "zone7");
  Serial2.print('&');
  Serial.println("Zone7");
}
void handleZone8() {
  server.send(200, "text/plain", "zone8");
  Serial2.print('*');
  Serial.println("Zone8");
}

void handleSrc1() {
  server.send(200, "text/plain", "src1");
  Serial.println("Src1");
  Serial2.print('1');
}
void handleSrc2() {
  server.send(200, "text/plain", "src2");
  Serial.println("Src2");
  Serial2.print('2');
}
void handleSrc3() {
  server.send(200, "text/plain", "src3");
  Serial.println("Src3");
  Serial2.print('3');
}
void handleSrc4() {
  server.send(200, "text/plain", "src4");
  Serial.println("Src4");
  Serial2.print('4');
}
void handleSrc5() {
  server.send(200, "text/plain", "src5");
  Serial.println("Src5");
  Serial2.print('5');
}
void handleSrc6() {
  server.send(200, "text/plain", "src6");
  Serial.println("Src6");
  Serial2.print('6');
}
void handleSrc7() {
  server.send(200, "text/plain", "src7");
  Serial.println("Src7");
  Serial2.print('7');
}
void handleSrc8() {
  server.send(200, "text/plain", "src8");
  Serial.println("Src8");
  Serial2.print('8');
}

void handleVol1() {
  server.send(200, "text/plain", "vol1");
  Serial.println("Vol Up");
  Serial2.print('V');
}
void handleVol2() {
  server.send(200, "text/plain", "vol2");
  Serial.println("Vol Dn");
  Serial2.print('v');
}
void handleVol3() {
  server.send(200, "text/plain", "vol3");
  Serial.println("Mute");
  Serial2.print('q');
}
void handleVol4() {
  server.send(200, "text/plain", "vol4");
  Serial.println("STFU");
  Serial2.print('Q');
}

void setup() {
  Serial.begin(115200);
  Serial2.begin(115200);

  pinMode(led1Pin, OUTPUT);
  pinMode(led2Pin, OUTPUT);
  digitalWrite(led1Pin, led1State);
 
  // WiFi Configuration & Init
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.print("Connecting To WiFi Network .");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(100);
  }
  // Connected Successfully
  Serial.println("\nConnected To The WiFi Network");
  Serial.print("Local ESP32 IP: ");
  Serial.println(WiFi.localIP());
  // Attach Server Handler Functions & Start WebServer
  server.on("/", handleRoot);
  server.on("/led1", handleLED);
  
  server.on("/zone1", handleZone1);
  server.on("/zone2", handleZone2);
  server.on("/zone3", handleZone3);
  server.on("/zone4", handleZone4);
  server.on("/zone5", handleZone5);
  server.on("/zone6", handleZone6);
  server.on("/zone7", handleZone7);
  server.on("/zone8", handleZone8); 

  server.on("/src1", handleSrc1);
  server.on("/src2", handleSrc2);
  server.on("/src3", handleSrc3);
  server.on("/src4", handleSrc4);
  server.on("/src5", handleSrc5);
  server.on("/src6", handleSrc6);
  server.on("/src7", handleSrc7);
  server.on("/src8", handleSrc8);

  server.on("/vol1", handleVol1);
  server.on("/vol2", handleVol2);
  server.on("/vol3", handleVol3);
  server.on("/vol4", handleVol4); 
 // server.on("/brightness", handleBrightness);
  server.begin();
}
 
void loop() {
  server.handleClient();
}
